from scontoolkit.interfaces.catalog import ICatalogService
from scontoolkit.models.dsp.catalog import Dataset
from uuid import uuid4
from scontoolkit.models.dsp.catalog import Dataset
from scontoolkit.models.dsp.low_level import Distribution, Offer
from ckanapi import RemoteCKAN
import os

# A Dataset MUST have at least one hasPolicy attribute that contain an Offer defining the Policy associated with the Dataset.
# A Dataset MUST hold at least one Distribution object in the distribution attribute.
# Each DataService object MUST HAVE at least one DataService which specifies where the distribution is obtained. Specifically, a DataService specifies the endpoint for initiating a Contract Negotiation and Transfer Process.
# A DataService.endpointURL property contains the URL of the service the Contract Negotiation endpoints extend. The endpoint's DSP version MUST be consistent with the version the Catalog object was served through.
# An Offer contains the following attributes:
#
# An Offer MUST have an @id that is a unique identifier.
# An Offer MUST be unique to a Dataset since the target of the Offer is derived from its enclosing context.
# Offers MUST NOT contain any target attributes. The value of the target attribute MUST be the Dataset ID. (Note: If the Offer is used in an enclosing Catalog or Dataset, there must not be any target attribute set.)
# 5.3.3 ERROR - Catalog Error

dummy_dataset = Dataset(
    id=f"urn:uuid:{uuid4()}",
    distribution=[
        Distribution(
            accessService="urn:uuid:dataservice-1",
            format="application/json",
            hasPolicy=[
                Offer(
                    id="urn:uuid:offer-1",
                    type="Offer",
                    permission=[],
                    profile=["https://w3id.org/idsa/code/EULicense"],
                    prohibition=[],
                    obligation=[]
                )
            ]
        )
    ],
    hasPolicy=[
        Offer(
            id="urn:uuid:offer-2",
            type="Offer",
            permission=[],
            profile=["https://w3id.org/idsa/code/EULicense"],
            prohibition=[],
            obligation=[]
        )
    ]
)


class CKANCatalogService(ICatalogService):
    def __init__(self):
        self._datasets = {}
        self.__ckan_url__ = os.getenv("sconext.ckan.url")
        self.__admin_key__ = os.getenv("sconext.ckan.admin_key")

        self.ckan = RemoteCKAN(self.__ckan_url__ , apikey=self.__admin_key__)

    def __retrieve_all_datasets__(self):
        orgs = self.ckan.action.organization_list()
        datasets = []
        for org in orgs:
            current = self.ckan.action.organization_show(id=org, include_datasets=True)
            datasets.extend(current['packages'])
        return datasets

    def get_catalog(self):
        dsets = self.__retrieve_all_datasets__()


    def create_dataset(self, dataset: Dataset) -> str:
        self._datasets[dataset.id] = dataset
        return dataset.id

    def get_dataset(self, dataset_id: str) -> Dataset:
        return self._datasets[dataset_id]

    def list_datasets(self) -> list[Dataset]:
        return list(self._datasets.values())

    def delete_dataset(self, dataset_id: str) -> bool:
        return self._datasets.pop(dataset_id, None) is not None
